@@ -5,12 +5,12 @@
 # Tested on Fedora Core 3/5/8, Debian Etch, RHEL 5/6.
 #
 # Cross toolchain and NSIS for Linux/i386/x86_64 can be fetched from
-#  http://www.ch-werner.de/xtools/crossmingw64-0.2-1.i386.rpm
-#  http://www.ch-werner.de/xtools/crossmingw64-0.2-1.x86_64.rpm
+#  http://www.ch-werner.de/xtools/crossmingw64-0.3-1.i386.rpm
+#  http://www.ch-werner.de/xtools/crossmingw64-0.3-1.x86_64.rpm
 #  http://www.ch-werner.de/xtools/nsis-2.37-1.i386.rpm
 # or
-#  http://www.ch-werner.de/xtools/crossmingw64-0.2.i386.tar.bz2
-#  http://www.ch-werner.de/xtools/crossmingw64-0.2.x86_64.tar.bz2
+#  http://www.ch-werner.de/xtools/crossmingw64-0.3.i386.tar.bz2
+#  http://www.ch-werner.de/xtools/crossmingw64-0.3.x86_64.tar.bz2
 #  http://www.ch-werner.de/xtools/nsis-2.37-1_i386.tar.gz
 
 # Some aspects of the build process can be controlled by shell variables:
@@ -23,8 +23,9 @@
 set -e
 
 VER2=2.8.17
-VER3=3.7.13
-VER3X=3071300
+VER3=3.7.16.1
+VER3X=3071601
+VERZ=1.2.7
 
 nov2=false
 if test -n "$NO_SQLITE2" ; then
@@ -58,6 +59,15 @@ if test -n "$WITH_SOURCES" ; then
     ADD_NSIS="$ADD_NSIS -DWITH_SOURCES"
 fi
 
+echo "=================="
+echo "Preparing zlib ..."
+echo "=================="
+test -r zlib-${VERZ}.tar.gz || \
+    wget -c http://zlib.net/zlib-${VERZ}.tar.gz || exit 1
+rm -rf zlib-${VERZ}
+tar xzf zlib-${VERZ}.tar.gz
+ln -sf zlib-${VERZ} zlib
+
 echo "===================="
 echo "Preparing sqlite ..."
 echo "===================="
@@ -344,7 +354,10 @@ EOD
 
 # same but new module libshell.c
 cp -p sqlite3/src/shell.c sqlite3/src/libshell.c
-patch sqlite3/src/libshell.c <<'EOD'
+test "$VER3" != "3.7.14" -a "$VER3" != "3.7.14.1" -a "$VER3" != "3.7.15" \
+  -a "$VER3" != "3.7.15.1" -a "$VER3" != "3.7.15.2" -a "$VER3" != "3.7.16" \
+  -a "$VER3" != "3.7.16.1" \
+  && patch sqlite3/src/libshell.c <<'EOD'
 --- sqlite3.orig/src/libshell.c  2007-01-08 23:40:05.000000000 +0100
 +++ sqlite3/src/libshell.c  2007-01-10 18:35:43.000000000 +0100
 @@ -21,6 +21,10 @@
@@ -387,6 +400,95 @@ patch sqlite3/src/libshell.c <<'EOD'
  #else
 EOD
 
+test "$VER3" = "3.7.14" -o "$VER3" = "3.7.14.1" \
+  && patch sqlite3/src/libshell.c <<'EOD'
+--- sqlite3.orig/src/libshell.c  2007-01-08 23:40:05.000000000 +0100
++++ sqlite3/src/libshell.c  2007-01-10 18:35:43.000000000 +0100
+@@ -21,6 +21,10 @@
+ #include <ctype.h>
+ #include <stdarg.h>
+
++#ifdef _WIN32
++# include <windows.h>
++#endif
++
+ #if !defined(_WIN32) && !defined(WIN32)
+ # include <signal.h>
+ # include <pwd.h>
+@@ -1774,7 +1778,7 @@
+   strcpy(continuePrompt,"   ...> ");
+ }
+ 
+-int main(int argc, char **argv){
++int sqlite3_main(int argc, char **argv){
+   char *zErrMsg = 0;
+   struct callback_data data;
+   const char *zInitFile = 0;
+@@ -1816,6 +1820,17 @@
+   if( i<argc ){
+     data.zDbFilename = argv[i++];
+   }else{
++#if defined(_WIN32) && !defined(__TINYC__)
++    static OPENFILENAME ofn;
++    static char zDbFn[1024];
++    ofn.lStructSize = sizeof(ofn);
++    ofn.lpstrFile = (LPTSTR) zDbFn;
++    ofn.nMaxFile = sizeof(zDbFn);
++    ofn.Flags = OFN_PATHMUSTEXIST | OFN_EXPLORER | OFN_NOCHANGEDIR;
++    if( GetOpenFileName(&ofn) ){
++      data.zDbFilename = zDbFn;
++    } else
++#endif
+ #ifndef SQLITE_OMIT_MEMORYDB
+     data.zDbFilename = ":memory:";
+ #else
+EOD
+
+test "$VER3" = "3.7.15" -o "$VER3" = "3.7.15.1" -o "$VER3" = "3.7.15.2" \
+  -o "$VER3" = "3.7.16" -o "$VER3" = "3.7.16.1" \
+  && patch sqlite3/src/libshell.c <<'EOD'
+--- sqlite3.orig/src/libshell.c  2012-12-12 14:42:10.000000000 +0100
++++ sqlite3/src/libshell.c  2012-12-13 12:14:57.000000000 +0100
+@@ -36,6 +36,10 @@
+ #include <ctype.h>
+ #include <stdarg.h>
+ 
++#ifdef _WIN32
++#include <windows.h>
++#endif
++
+ #if !defined(_WIN32) && !defined(WIN32)
+ # include <signal.h>
+ # if !defined(__RTP__) && !defined(_WRS_KERNEL)
+@@ -2894,7 +2898,7 @@
+   return argv[i];
+ }
+ 
+-int main(int argc, char **argv){
++int sqlite3_main(int argc, char **argv){
+   char *zErrMsg = 0;
+   struct callback_data data;
+   const char *zInitFile = 0;
+@@ -2996,6 +3000,17 @@
+     }
+   }
+   if( data.zDbFilename==0 ){
++#if defined(_WIN32) && !defined(__TINYC__)
++    static OPENFILENAME ofn;
++    static char zDbFn[1024];
++    ofn.lStructSize = sizeof(ofn);
++    ofn.lpstrFile = (LPTSTR) zDbFn;
++    ofn.nMaxFile = sizeof(zDbFn);
++    ofn.Flags = OFN_PATHMUSTEXIST | OFN_EXPLORER | OFN_NOCHANGEDIR;
++    if( GetOpenFileName(&ofn) ){
++      data.zDbFilename = zDbFn;
++    } else
++#endif
+ #ifndef SQLITE_OMIT_MEMORYDB
+     data.zDbFilename = ":memory:";
+ #else
+EOD
+
 rm -f sqlite3/src/minshell.c
 touch sqlite3/src/minshell.c
 patch sqlite3/src/minshell.c <<'EOD'
@@ -452,6 +554,9 @@ test "$VER3" != "3.6.15" -a "$VER3" != "3.6.16" -a "$VER3" != "3.6.17" \
   -a "$VER3" != "3.7.7" -a "$VER3" != "3.7.7.1" -a "$VER3" != "3.7.8" \
   -a "$VER3" != "3.7.9" -a "$VER3" != "3.7.10" -a "$VER3" != "3.7.11" \
   -a "$VER3" != "3.7.12" -a "$VER3" != "3.7.12.1" -a "$VER3" != "3.7.13" \
+  -a "$VER3" != "3.7.14" -a "$VER3" != "3.7.14.1" -a "$VER3" != "3.7.15" \
+  -a "$VER3" != "3.7.15.1" -a "$VER3" != "3.7.15.2" -a "$VER3" != "3.7.16" \
+  -a "$VER3" != "3.7.16.1" \
   && patch -d sqlite3 -p1 <<'EOD'
 diff -u sqlite3.orig/src/build.c sqlite3/src/build.c
 --- sqlite3.orig/src/build.c	2007-01-09 14:53:04.000000000 +0100
@@ -516,7 +621,7 @@ diff -u sqlite3.orig/src/tclsqlite.c sqlite3/src/tclsqlite.c
 +++ sqlite3/src/tclsqlite.c	2007-04-10 07:47:49.000000000 +0200
 @@ -14,6 +14,7 @@
  **
- ** $Id: mingw64-cross-build.sh,v 1.28 2012/06/24 09:36:50 chw Exp chw $
+ ** $Id: mingw64-cross-build.sh,v 1.37 2013/03/30 05:16:56 chw Exp chw $
  */
 +#ifndef NO_TCL     /* Omit this whole file if TCL is unavailable */
  #include "tcl.h"
@@ -671,6 +776,9 @@ test "$VER3" != "3.6.21" -a "$VER3" != "3.6.22" -a "$VER3" != "3.6.23" \
   -a "$VER3" != "3.7.7" -a "$VER3" != "3.7.7.1" -a "$VER3" != "3.7.8" \
   -a "$VER3" != "3.7.9" -a "$VER3" != "3.7.10" -a "$VER3" != "3.7.11" \
   -a "$VER3" != "3.7.12" -a "$VER3" != "3.7.12.1" -a "$VER3" != "3.7.13" \
+  -a "$VER3" != "3.7.14" -a "$VER3" != "3.7.14.1" -a "$VER3" != "3.7.15" \
+  -a "$VER3" != "3.7.15.1" -a "$VER3" != "3.7.15.2" -a "$VER3" != "3.7.16" \
+  -a "$VER3" != "3.7.16.1" \
   && patch -d sqlite3 -p1 <<'EOD'
 --- sqlite3.orig/ext/fts3/fts3.c 2008-02-02 17:24:34.000000000 +0100
 +++ sqlite3/ext/fts3/fts3.c      2008-03-16 11:29:02.000000000 +0100
@@ -810,7 +918,9 @@ patch -d sqlite3 -p1 <<'EOD'
 EOD
 test "$VER3" != "3.7.8" -a "$VER3" != "3.7.9" -a "$VER3" != "3.7.10" \
   -a "$VER3" != "3.7.11" -a "$VER3" != "3.7.12" -a "$VER3" != "3.7.12.1" \
-  -a "$VER3" != "3.7.13" \
+  -a "$VER3" != "3.7.13" -a "$VER3" != "3.7.14" -a "$VER3" != "3.7.14.1" \
+  -a "$VER3" != "3.7.15" -a "$VER3" != "3.7.15.1" -a "$VER3" != "3.7.15.2" \
+  -a "$VER3" != "3.7.16" -a "$VER3" != "3.7.16.1" \
   && patch -d sqlite3 -p1 <<'EOD'
 --- sqlite3.orig/ext/fts3/fts3_hash.c    2007-11-24 01:41:52.000000000 +0100
 +++ sqlite3/ext/fts3/fts3_hash.c 2008-03-16 11:39:57.000000000 +0100
@@ -1171,6 +1281,9 @@ test "$VER3" = "3.7.3" -o "$VER3" = "3.7.4" -o "$VER3" = "3.7.5" \
   -o "$VER3" = "3.7.7" -o "$VER3" = "3.7.7.1" -o "$VER3" = "3.7.8" \
   -o "$VER3" = "3.7.9" -o "$VER3" = "3.7.10" -o "$VER3" = "3.7.11" \
   -o "$VER3" = "3.7.12" -o "$VER3" = "3.7.12.1" -o "$VER3" = "3.7.13" \
+  -o "$VER3" = "3.7.14" -o "$VER3" = "3.7.14.1" -o "$VER3" = "3.7.15" \
+  -o "$VER3" = "3.7.15.1" -o "$VER3" = "3.7.15.2" -o "$VER3" = "3.7.16" \
+  -o "$VER3" = "3.7.16.1" \
   && patch -d sqlite3 -p1 <<'EOD'
 --- sqlite3.orig/ext/rtree/rtree.c	2010-10-16 10:53:54.000000000 +0200
 +++ sqlite3/ext/rtree/rtree.c	2010-10-16 11:12:32.000000000 +0200
@@ -1199,6 +1312,7 @@ test "$VER3" = "3.7.6.1" -o "$VER3" = "3.7.6.2" -o "$VER3" = "3.7.6.3" \
   -o "$VER3" = "3.7.7" -o "$VER3" = "3.7.7.1" -o "$VER3" = "3.7.8" \
   -o "$VER3" = "3.7.9" -o "$VER3" = "3.7.10" -o "$VER3" = "3.7.11" \
   -o "$VER3" = "3.7.12" -o "$VER3" = "3.7.12.1" -o "$VER3" = "3.7.13" \
+  -o "$VER3" = "3.7.14" -o "$VER3" = "3.7.14.1" \
   && patch -d sqlite3 -p1 <<'EOD'
 --- sqlite3.orig/src/shell.c	2011-05-19 15:34:57.000000000 +0200
 +++ sqlite3/src/shell.c	2011-06-09 13:36:13.000000000 +0200
@@ -1216,6 +1330,9 @@ EOD
 test "$VER3" = "3.7.7" -o "$VER3" = "3.7.7.1" -o "$VER3" = "3.7.8" \
   -o "$VER3" = "3.7.9" -o "$VER3" = "3.7.10" -o "$VER3" = "3.7.11" \
   -o "$VER3" = "3.7.12" -o "$VER3" = "3.7.12.1" -o "$VER3" = "3.7.13" \
+  -o "$VER3" = "3.7.14" -o "$VER3" = "3.7.14.1" -o "$VER3" = "3.7.15" \
+  -o "$VER3" = "3.7.15.1" -o "$VER3" = "3.7.15.2" -o "$VER3" = "3.7.16" \
+  -o "$VER3" = "3.7.16.1" \
   && patch -d sqlite3 -p1 <<'EOD'
 --- sqlite3.orig/ext/fts3/fts3_aux.c	2011-06-24 09:06:08.000000000 +0200
 +++ sqlite3/ext/fts3/fts3_aux.c	2011-06-25 06:44:08.000000000 +0200
@@ -1258,7 +1375,9 @@ test "$VER3" = "3.7.7" -o "$VER3" = "3.7.7.1" \
 EOD
 test "$VER3" = "3.7.8" -o "$VER3" = "3.7.9" -o "$VER3" = "3.7.10" \
   -o "$VER3" = "3.7.11" -o "$VER3" = "3.7.12" -o "$VER3" = "3.7.12.1" \
-  -o "$VER3" = "3.7.13" \
+  -o "$VER3" = "3.7.13" -o "$VER3" = "3.7.14" -o "$VER3" = "3.7.14.1" \
+  -o "$VER3" = "3.7.15" -o "$VER3" = "3.7.15.1" -o "$VER3" = "3.7.15.2" \
+  -o "$VER3" = "3.7.16" -o "$VER3" = "3.7.16.1" \
   && patch -d sqlite3 -p1 <<'EOD'
 --- sqlite3.orig/ext/fts3/fts3.c	2011-09-19 20:46:52.000000000 +0200
 +++ sqlite3/ext/fts3/fts3.c	2011-09-20 09:47:40.000000000 +0200
@@ -1286,6 +1405,9 @@ EOD
 test "$VER3" = "3.7.7" -o "$VER3" = "3.7.7.1" -o "$VER3" = "3.7.8" \
   -o "$VER3" = "3.7.9" -o "$VER3" = "3.7.10" -o "$VER3" = "3.7.11" \
   -o "$VER3" = "3.7.12" -o "$VER3" = "3.7.12.1" -o "$VER3" = "3.7.13" \
+  -o "$VER3" = "3.7.14" -o "$VER3" = "3.7.14.1" -o "$VER3" = "3.7.15" \
+  -o "$VER3" = "3.7.15.1" -o "$VER3" = "3.7.15.2" -o "$VER3" = "3.7.16" \
+  -o "$VER3" = "3.7.16.1" \
   && patch -d sqlite3 -p1 <<'EOD'
 --- sqlite3.orig/ext/fts3/fts3_expr.c	2011-06-24 09:06:08.000000000 +0200
 +++ sqlite3/ext/fts3/fts3_expr.c	2011-06-25 06:47:00.000000000 +0200
@@ -1404,7 +1526,8 @@ test "$VER3" = "3.7.7" -o "$VER3" = "3.7.7.1" \
  /*
 EOD
 test "$VER3" = "3.7.11" -o "$VER3" = "3.7.12" -o "$VER3" = "3.7.12.1" \
-  -o "$VER3" = "3.7.13" \
+  -o "$VER3" = "3.7.13" -o "$VER3" = "3.7.14" -o "$VER3" = "3.7.14.1" \
+  -o "$VER3" = "3.7.15" -o "$VER3" = "3.7.15.1" -o "$VER3" = "3.7.15.2" \
   && patch -d sqlite3 -p1 <<'EOD'
 --- sqlite3.orig/src/sqlite3ext.h     2012-03-22 20:13:33.000000000 +0100
 +++ sqlite3/src/sqlite3ext.h  2012-03-22 20:13:57.000000000 +0100
@@ -1455,6 +1578,12 @@ if test -n "$SQLITE_DLLS" ; then
     $nov2 || make -C sqlite -f ../mf-sqlite.mingw64-cross sqlite.dll
 fi
 
+echo "================="
+echo "Building zlib ..."
+echo "================="
+
+make -C zlib -f ../mf-zlib.mingw64-cross all
+
 echo "=========================="
 echo "Building SQLite 2 ... UTF8"
 echo "=========================="
